<!DOCTYPE html>
<html>
<head>
  <meta charset=utf-8>
  <title>Combine error</title>
  <style>
    body {
      flex: auto;
      font-family: sans-serif;
    }
    .container {
      max-width: 800px;
      margin: 30px auto;
    }
    code.error {
      color: red;
    }
    code.short {
      color: red;
      background: #ffebeb;
      padding: 2px 8px;
      border-radius: 5px;
    }
    pre {
      background: #f5f5f5;
      padding: 10px;
      border-radius: 7px;
      white-space: pre-wrap;
    }
    p {
      color: #989898;
    }
  </style>
</head>
<body>
  <div class="container">
    <h1>Error! <code class="short">{{ error }}</code></h1>
    {% if error.lineno is defined %}<p>{{ relative_path }}:{{ error.lineno }}</p>{% endif %}
    <code class="template"><pre>{{ context_lines }}</pre></code>
    <code class="error"><pre>{{ excinfo }}</pre></code>
    <p style="margin-top: 40px;">This screen is only shown in development. This won't be shown in production since the entire build would fail.</p>
  </div>
</body>
</html>
